//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 LimeGreen
#property indicator_color2 Orange

extern int  LagLookBack     = 20;
extern int  LagMedian       = 5;
extern int  LagPrice        = PRICE_MEDIAN;
extern int  MaPeriod        = 89;
extern int  MaMethod        = MODE_LWMA;
extern int  MaPrice         = PRICE_CLOSE;
extern bool alertsOn        = false;
extern bool alertsOnCurrent = false;
extern bool alertsMessage   = true;
extern bool alertsSound     = false;
extern bool alertsEmail     = false;

double filter[];
double diff[];
double ma[];
double cross[];
double sortDiff[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init() 
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,filter);
   SetIndexBuffer(1,ma);
   SetIndexBuffer(2,diff);
   SetIndexBuffer(3,cross);
      ArrayResize(sortDiff,LagMedian);
   return (0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         
         //
         //
         //
         //
         //

   for (int i=limit; i>=0; i--)   
   {
      double price   = iMA(NULL,0,1,0,MODE_SMA,LagPrice,i);
             diff[i] = MathAbs(price - filter[i+1]);
      double hi    = diff[ArrayMaximum(diff,LagLookBack,i)];
      double lo    = diff[ArrayMinimum(diff,LagLookBack,i)];
      double alpha = 0;
         if (hi!=lo)    
         {
            for (int j=0; j<LagMedian; j++) sortDiff[j] = (diff[i+j]-lo)/(hi-lo);
               ArraySort(sortDiff,WHOLE_ARRAY,0,MODE_ASCEND);
                  if (MathMod(LagMedian,2.0) != 0)   alpha = sortDiff[LagMedian/2];         
                  else                               alpha = (sortDiff[LagMedian/2]+sortDiff[(LagMedian/2)-1])/2;
         }
      filter[i] = iLaGuerreFilter(price, 1-alpha, i, 0);
      ma[i]     = iMA(NULL,0,MaPeriod,0,MaMethod,MaPrice,i);
      cross[i]  = cross[i+1];
         if (filter[i]>ma[i]) cross[i] =  1;
         if (filter[i]<ma[i]) cross[i] = -1;
   }
   manageAlerts();
   return (0);   
}


//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

double workLagFil[][4];
double iLaGuerreFilter(double price, double gamma, int i, int instanceNo=0)
{
   if (ArrayRange(workLagFil,0)!=Bars) ArrayResize(workLagFil,Bars); i=Bars-i-1; instanceNo*=4;
   if (gamma<=0) return(price);

   //
   //
   //
   //
   //
      
   workLagFil[i][instanceNo+0] = (1.0 - gamma)*price                                                + gamma*workLagFil[i-1][instanceNo+0];
	workLagFil[i][instanceNo+1] = -gamma*workLagFil[i][instanceNo+0] + workLagFil[i-1][instanceNo+0] + gamma*workLagFil[i-1][instanceNo+1];
	workLagFil[i][instanceNo+2] = -gamma*workLagFil[i][instanceNo+1] + workLagFil[i-1][instanceNo+1] + gamma*workLagFil[i-1][instanceNo+2];
	workLagFil[i][instanceNo+3] = -gamma*workLagFil[i][instanceNo+2] + workLagFil[i-1][instanceNo+2] + gamma*workLagFil[i-1][instanceNo+3];

   //
   //
   //
   //
   //
 
   return((workLagFil[i][instanceNo+0]+2.0*workLagFil[i][instanceNo+1]+2.0*workLagFil[i][instanceNo+2]+workLagFil[i][instanceNo+3])/6.0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (cross[whichBar] != cross[whichBar+1])
      {
         if (cross[whichBar] ==  1) doAlert(whichBar,"adaptive laguerre crossed ma up");
         if (cross[whichBar] == -1) doAlert(whichBar,"adaptive laguerre crossed ma down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" adaptive laguerre + ma crosses",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

